@@ -37,7 +37,20 @@ dialog_response (GtkWidget *widget,
 		 gint       response_id,
 		 GConfChangeSet *changeset)
 {
-	gtk_main_quit ();
+	if (response_id == GTK_RESPONSE_HELP) {
+		GError *error = NULL;
+
+		/* TODO : get this written */
+		gnome_help_display_desktop (NULL,
+			"accessibility-keyboard",
+			"accessibility-keyboard.xml",
+			"CONFIGURATION", &error);
+		if (error) {
+			g_warning ("help error: %s\n", error->message);
+			g_error_free (error);
+		}
+	} else
+		gtk_main_quit ();
 }
 
 int
@@ -45,7 +58,6 @@ main (int argc, char **argv)
 {
 	GtkWidget *dialog;
 	GConfChangeSet *changeset;
-
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
@@ -54,7 +66,7 @@ main (int argc, char **argv)
 			    GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 			    NULL);
 	activate_settings_daemon ();
-
+	
 	changeset = NULL;
 	dialog = setup_accessX_dialog (changeset, TRUE);
 	g_signal_connect (G_OBJECT (dialog),
