@@ -463,6 +463,102 @@ gconf_peditor_new_boolean (GConfChangeSet *changeset,
 	return peditor;
 }
 
+static void
+peditor_integer_value_changed (GConfClient         *client,
+			       guint                cnxn_id,
+			       GConfEntry          *entry,
+			       GConfPropertyEditor *peditor) 
+{
+	GConfValue *value, *value_wid;
+	const char *entry_current_text;
+	int         entry_current_integer;
+
+	if (peditor->p->changeset != NULL)
+		gconf_change_set_remove (peditor->p->changeset, peditor->p->key);
+
+	value = gconf_entry_get_value (entry);
+
+	if (value != NULL) {
+		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
+		entry_current_text = gtk_entry_get_text (GTK_ENTRY (peditor->p->ui_control));
+		entry_current_integer = strtol (entry_current_text, NULL, 10);
+		if (entry_current_integer != gconf_value_get_int (value)) {
+			char *buf = g_strdup_printf ("%d", gconf_value_get_int (value_wid));
+			gtk_entry_set_text (GTK_ENTRY (peditor->p->ui_control), buf);
+			g_free (buf);
+		}
+		gconf_value_free (value_wid);
+	}
+}
+
+static void
+peditor_integer_widget_changed (GConfPropertyEditor *peditor,
+				GtkEntry            *entry)
+{
+	GConfValue *value, *value_wid;
+	  
+	if (!peditor->p->inited) return;
+
+	value_wid = gconf_value_new (GCONF_VALUE_INT);
+
+	gconf_value_set_int (value_wid, strtol (gtk_entry_get_text (entry), NULL, 10));
+	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
+
+	peditor_set_gconf_value (peditor, peditor->p->key, value);
+
+	g_signal_emit (peditor, peditor_signals[VALUE_CHANGED], 0, peditor->p->key, value);
+	gconf_value_free (value_wid);
+	gconf_value_free (value);
+}
+
+static GObject *
+gconf_peditor_new_integer_valist (GConfChangeSet *changeset,
+				  gchar          *key,
+				  GtkWidget      *entry,
+				  gchar          *first_property_name,
+				  va_list         var_args)
+{
+	GObject *peditor;
+
+	peditor = gconf_peditor_new
+		(key,
+		 (GConfClientNotifyFunc) peditor_integer_value_changed,
+		 changeset,
+		 G_OBJECT (entry),
+		 first_property_name,
+		 var_args, NULL);
+
+	g_signal_connect_swapped (G_OBJECT (entry), "changed",
+				  (GCallback) peditor_integer_widget_changed, peditor);
+
+	return peditor;
+}
+
+GObject *
+gconf_peditor_new_integer (GConfChangeSet *changeset,
+			   gchar          *key,
+			   GtkWidget      *entry,
+			   gchar          *first_property_name,
+			   ...)
+{
+	GObject *peditor;
+	va_list var_args;
+
+	g_return_val_if_fail (key != NULL, NULL);
+	g_return_val_if_fail (entry != NULL, NULL);
+	g_return_val_if_fail (GTK_IS_ENTRY (entry), NULL);
+
+	va_start (var_args, first_property_name);
+
+	peditor = gconf_peditor_new_integer_valist
+		(changeset, key, entry,
+		 first_property_name, var_args);
+
+	va_end (var_args);
+
+	return peditor;
+}
+
 static void
 peditor_string_value_changed (GConfClient         *client,
 			      guint                cnxn_id,
@@ -663,28 +759,23 @@ gconf_peditor_new_color (GConfChangeSet *changeset,
 }
 
 static int
-peditor_enum_int_from_string (GType type, const gchar *str, gboolean *use_nick)
+peditor_enum_int_from_string (GType type, const gchar *str, gboolean use_nick)
 {
 	GEnumClass *klass;
 	GEnumValue *val;
 	int ret = -1;
 
-	if (use_nick)
-		*use_nick = FALSE;
-	
 	klass = g_type_class_ref (type);
-	val = g_enum_get_value_by_name (klass, str);
-	if (!val)
-	{
+	if (use_nick)
 		val = g_enum_get_value_by_nick (klass, str);
-		if (use_nick)
-			*use_nick = TRUE;
-	}
-	if (val)
-		ret = val->value;
-
+	else
+		val = g_enum_get_value_by_name (klass, str);
+	
 	g_type_class_unref (klass);
 
+	if (val)
+		ret = val->value;
+	
 	return ret;
 }
 
@@ -725,7 +816,7 @@ peditor_enum_conv_to_widget (GConfPropertyEditor *peditor,
 	
 	index = peditor_enum_int_from_string (data->enum_type,
 		    		    	      gconf_value_get_string (value),
-					      &data->use_nick);
+					      data->use_nick);
 	
 	gconf_value_set_int (ret, index);
 
@@ -826,6 +917,7 @@ gconf_peditor_new_select_menu_with_enum	(GConfChangeSet *changeset,
 					 gchar 	        *key,
 					 GtkWidget      *option_menu,
 					 GType          enum_type,
+					 gboolean	use_nick,
 					 gchar          *first_property_name,
 					 ...)
 {
@@ -840,6 +932,7 @@ gconf_peditor_new_select_menu_with_enum	(GConfChangeSet *changeset,
 
 	data = g_new0 (GConfPropertyEditorEnumData, 1);
 	data->enum_type = enum_type;
+	data->use_nick = use_nick;
 
 	va_start (var_args, first_property_name);
 
@@ -1041,7 +1134,7 @@ guard_get_bool (GConfPropertyEditor *peditor, const GConfValue *value)
 	else
 	{
 		GConfPropertyEditorEnumData *data = peditor->p->data;
-		int index = peditor_enum_int_from_string (data->enum_type, gconf_value_get_string (value), &data->use_nick);
+		int index = peditor_enum_int_from_string (data->enum_type, gconf_value_get_string (value), data->use_nick);
 		return (index != data->enum_val_false);
 	}
 }
@@ -1102,20 +1195,71 @@ gconf_value_float_to_int (const GConfValue *value)
 }
 
 static gint
-peditor_font_get_size (const gchar *font_name)
+peditor_font_get_size (const gchar *font_str)
 {
 	PangoFontDescription *desc;
 	int size;
 	
-	g_return_val_if_fail (font_name != NULL, -1);
+	g_return_val_if_fail (font_str != NULL, -1);
 	
-	desc = pango_font_description_from_string (font_name);
+	desc = pango_font_description_from_string (font_str);
 	size = pango_font_description_get_size (desc);
 	pango_font_description_free (desc);
 
 	return size / PANGO_SCALE;
 }
 
+static gchar* 
+peditor_font_get_name (const gchar *font_str)
+{
+	PangoFontDescription *desc;
+	gchar *name;
+
+	g_return_val_if_fail (font_str != NULL, NULL);
+	
+	desc = pango_font_description_from_string (font_str);
+	pango_font_description_unset_fields (desc, PANGO_FONT_MASK_SIZE);
+	name = pango_font_description_to_string (desc);
+	pango_font_description_free (desc);
+
+	return name;
+}
+
+static void
+peditor_font_merge_setting (GnomeFontPicker *font_picker, GConfValue *value, GConfPEditorFontType font_type)
+{
+	PangoFontDescription *desc;
+	gchar *font_str;
+	
+	g_return_if_fail (GNOME_IS_FONT_PICKER (font_picker));
+	g_return_if_fail (value != NULL);
+	
+	
+	desc = pango_font_description_from_string (gnome_font_picker_get_font_name (font_picker));
+	
+	if (font_type == PEDITOR_FONT_SIZE)
+	{
+		pango_font_description_set_size (desc, gconf_value_get_int (value) * PANGO_SCALE);
+	}
+	else
+	{
+		PangoFontDescription *new_desc;
+		new_desc = pango_font_description_from_string (gconf_value_get_string (value));
+		if (font_type == PEDITOR_FONT_NAME)
+			pango_font_description_set_size (new_desc, pango_font_description_get_size (desc));
+		pango_font_description_free (desc);
+		desc = new_desc;
+	}
+
+	font_str = pango_font_description_to_string (desc);
+	g_object_set (G_OBJECT (font_picker),
+		      "font_name", font_str,
+		      "label-font-size", pango_font_description_get_size (desc) / PANGO_SCALE,
+		      NULL);
+	g_free (font_str);
+	pango_font_description_free (desc);
+}
+
 static void
 peditor_font_value_changed (GConfClient         *client,
 			     guint                cnxn_id,
@@ -1123,40 +1267,58 @@ peditor_font_value_changed (GConfClient         *client,
 			     GConfPropertyEditor *peditor) 
 {
 	GConfValue *value, *value_wid;
+	GConfPEditorFontType font_type;
 
 	if (peditor->p->changeset != NULL)
 		gconf_change_set_remove (peditor->p->changeset, peditor->p->key);
 
+	font_type = GPOINTER_TO_UINT (peditor->p->data);
 	value = gconf_entry_get_value (entry);
 
 	if (value != NULL) {
-		const gchar *font_name;
-
 		value_wid = peditor->p->conv_to_widget_cb (peditor, value);
-		font_name = gconf_value_get_string (value_wid);
-		g_object_set (G_OBJECT (peditor->p->ui_control),
-			      "font_name", font_name,
-			      "label-font-size", peditor_font_get_size (font_name),
-			      NULL);
+		peditor_font_merge_setting (GNOME_FONT_PICKER (peditor->p->ui_control), value_wid, font_type);
 		gconf_value_free (value_wid);
 	}
 }
 
 static void
 peditor_font_widget_changed (GConfPropertyEditor *peditor,
-			      gchar              *font_name,
+			      gchar              *font_str,
 			      GnomeFontPicker    *font_picker)
 {
-	GConfValue *value, *value_wid;
+	GConfValue *value, *value_wid = NULL;
+	GConfPEditorFontType font_type;
+	gchar *font_name;
+	int font_size;
 
 	if (!peditor->p->inited) return;
 
+	font_type = GPOINTER_TO_UINT (peditor->p->data);
+
+	font_size = peditor_font_get_size (font_str);
 	g_object_set (G_OBJECT (peditor->p->ui_control),
-		      "label-font-size", peditor_font_get_size (font_name),
+		      "label-font-size", font_size,
 		      NULL);
 	
-	value_wid = gconf_value_new (GCONF_VALUE_STRING);
-	gconf_value_set_string (value_wid, font_name);
+	switch (font_type)
+	{
+		case PEDITOR_FONT_NAME:
+			value_wid = gconf_value_new (GCONF_VALUE_STRING);
+			font_name = peditor_font_get_name (font_str);
+			gconf_value_set_string (value_wid, font_name);
+			g_free (font_name);
+			break;
+		case PEDITOR_FONT_SIZE:
+			value_wid = gconf_value_new (GCONF_VALUE_INT);
+			gconf_value_set_int (value_wid, font_size);
+			break;
+		case PEDITOR_FONT_COMBINED:
+			value_wid = gconf_value_new (GCONF_VALUE_STRING);
+			gconf_value_set_string (value_wid, font_str);
+			break;
+	}
+	
 	value = peditor->p->conv_from_widget_cb (peditor, value_wid);
 
 	peditor_set_gconf_value (peditor, peditor->p->key, value);
@@ -1170,6 +1332,7 @@ GObject *
 gconf_peditor_new_font (GConfChangeSet *changeset,
 			gchar *key,
 			GtkWidget *font_picker,
+			GConfPEditorFontType font_type,
 			gchar *first_property_name,
 			...)
 {
@@ -1188,7 +1351,9 @@ gconf_peditor_new_font (GConfChangeSet *changeset,
 		 changeset,
 		 G_OBJECT (font_picker),
 		 first_property_name,
-		 var_args, NULL);
+		 var_args,
+		 "data", GUINT_TO_POINTER (font_type),
+		 NULL);
 
 	va_end (var_args);
 
@@ -1213,7 +1378,7 @@ peditor_enum_toggle_conv_to_widget (GConfPropertyEditor *peditor,
 	
 	index = peditor_enum_int_from_string (data->enum_type,
 		    		    	      gconf_value_get_string (value),
-					      &data->use_nick);
+					      data->use_nick);
 	gconf_value_set_bool (ret, (index != data->enum_val_false));
 
 	return ret;
@@ -1251,6 +1416,7 @@ gconf_peditor_new_enum_toggle  (GConfChangeSet 	 *changeset,
 				GType			 enum_type,
 				GConfPEditorGetValueFn   val_true_fn,
 				guint			 val_false,
+				gboolean		 use_nick,
 				gpointer		 data,
 				gchar 			 *first_property_name,
 				...)
@@ -1268,6 +1434,7 @@ gconf_peditor_new_enum_toggle  (GConfChangeSet 	 *changeset,
 	enum_data->enum_val_true_fn = val_true_fn;
 	enum_data->enum_val_true_fn_data = data;
 	enum_data->enum_val_false = val_false;
+	enum_data->use_nick = use_nick;
 
 	va_start (var_args, first_property_name);
 
@@ -1402,7 +1569,8 @@ peditor_image_clicked_cb (GConfPropertyEditor *peditor, GtkButton *button)
 	GtkWidget *fsel;
 
 	fsel = preview_file_selection_new (_("Please select an image."), TRUE);
-	
+	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
+
 	/* need the current filename */
 	if (peditor->p->changeset)
 		gconf_change_set_check_value (peditor->p->changeset, peditor->p->key, &value);
@@ -1505,6 +1673,7 @@ gconf_peditor_new_select_radio_with_enum (GConfChangeSet *changeset,
 					  gchar		 *key,
 					  GSList 	 *radio_group,
 					  GType 	 enum_type,
+					  gboolean	 use_nick,
 					  gchar          *first_property_name,
 					  ...)
 {
@@ -1521,6 +1690,7 @@ gconf_peditor_new_select_radio_with_enum (GConfChangeSet *changeset,
 
 	enum_data = g_new0 (GConfPropertyEditorEnumData, 1);
 	enum_data->enum_type = enum_type;
+	enum_data->use_nick = use_nick;
 
 	first_button = GTK_RADIO_BUTTON (radio_group->data);
 
