@@ -34,7 +34,6 @@
 #include <libgnomevfs/gnome-vfs-application-registry.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <gconf/gconf-client.h>
-#include <ctype.h>
 
 #include "libuuid/uuid.h"
 
@@ -333,7 +332,8 @@ get_gconf_base_name (MimeCategoryInfo *category)
 	tmp1 = mime_category_info_get_full_name (category);
 
 	for (tmp = tmp1; *tmp != '\0'; tmp++)
-		if (isspace (*tmp) || *tmp == '(' || *tmp == ')') *tmp = '-';
+		if (g_ascii_isspace (*tmp) || *tmp == '(' || *tmp == ')')
+			*tmp = '-';
 
 	tmp = g_strconcat ("/desktop/gnome/file-types-categories/", tmp1, NULL);
 
@@ -880,6 +880,7 @@ static MimeCategoryInfo *
 get_category (const gchar *category_name, const gchar *category_desc, GtkTreeModel *model)
 {
 	ModelEntry *current, *child;
+	gchar **cf = NULL, **df = NULL;
 	gchar **categories = NULL, **desc_categories = NULL;
 	int i;
 
@@ -887,11 +888,9 @@ get_category (const gchar *category_name, const gchar *category_desc, GtkTreeMod
 		return NULL;
 
 	if (category_name != NULL)
-		categories = g_strsplit (category_name, "/", -1);
-
+		categories = cf = g_strsplit (category_name, "/", -1);
 	if (category_desc != NULL)
-		desc_categories = g_strsplit (category_desc, "/", -1);
-
+		desc_categories = df = g_strsplit (category_desc, "/", -1);
 	if (category_name == NULL)
 		categories = desc_categories;
 	else if (category_desc == NULL)
@@ -904,7 +903,7 @@ get_category (const gchar *category_name, const gchar *category_desc, GtkTreeMod
 			if (child->type != MODEL_ENTRY_CATEGORY)
 				continue;
 
-			if (!strcmp ((category_name == NULL) ?
+			if (!g_ascii_strcasecmp ((category_name == NULL) ?
 				     MIME_CATEGORY_INFO (child)->description :
 				     MIME_CATEGORY_INFO (child)->name,
 				     categories[i]))
@@ -920,8 +919,8 @@ get_category (const gchar *category_name, const gchar *category_desc, GtkTreeMod
 		current = child;
 	}
 
-	g_strfreev (categories);
-	g_strfreev (desc_categories);
+	g_strfreev (cf);
+	g_strfreev (df);
 
 	return MIME_CATEGORY_INFO (current);
 }
