@@ -27,6 +27,7 @@
 #  include <config.h>
 #endif
 
+#include <string.h>
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
@@ -34,6 +35,8 @@
 
 #include "capplet-util.h"
 #include "gconf-property-editor.h"
+#include "activate-settings-daemon.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -76,7 +79,7 @@ gint double_click_state = DOUBLE_CLICK_TEST_OFF;
  * have a few routines to convert from whatever the gconf key is to our range.
  */
 static GConfValue *
-double_click_from_gconf (const GConfValue *value)
+double_click_from_gconf (GConfPropertyEditor *peditor, const GConfValue *value)
 {
 	GConfValue *new_value;
 
@@ -86,7 +89,7 @@ double_click_from_gconf (const GConfValue *value)
 }
 
 static GConfValue *
-double_click_to_gconf (const GConfValue *value)
+double_click_to_gconf (GConfPropertyEditor *peditor, const GConfValue *value)
 {
 	GConfValue *new_value;
 
@@ -96,7 +99,8 @@ double_click_to_gconf (const GConfValue *value)
 }
 
 static GConfValue *
-motion_acceleration_from_gconf (const GConfValue *value)
+motion_acceleration_from_gconf (GConfPropertyEditor *peditor, 
+				const GConfValue *value)
 {
 	GConfValue *new_value;
 	gfloat motion_acceleration;
@@ -113,7 +117,8 @@ motion_acceleration_from_gconf (const GConfValue *value)
 }
 
 static GConfValue *
-motion_acceleration_to_gconf (const GConfValue *value)
+motion_acceleration_to_gconf (GConfPropertyEditor *peditor, 
+			      const GConfValue *value)
 {
 	GConfValue *new_value;
 	gfloat motion_acceleration;
@@ -130,7 +135,8 @@ motion_acceleration_to_gconf (const GConfValue *value)
 }
 
 static GConfValue *
-threshold_from_gconf (const GConfValue *value)
+threshold_from_gconf (GConfPropertyEditor *peditor, 
+		      const GConfValue *value)
 {
 	GConfValue *new_value;
 
@@ -139,6 +145,13 @@ threshold_from_gconf (const GConfValue *value)
 	return new_value;
 }
 
+static GConfValue *
+threshold_to_gconf (GConfPropertyEditor *peditor, 
+		    const GConfValue *value)
+{
+	return gconf_value_float_to_int (value);
+}
+
 /* Retrieve legacy settings */
 
 static void
@@ -185,7 +198,7 @@ drawing_area_button_press_event (GtkWidget      *widget,
 	if (event->type != GDK_BUTTON_PRESS)
 		return FALSE;
 
-	if (!gconf_change_set_check_value (changeset, DOUBLE_CLICK_KEY, &value)) 
+	if (!(changeset && gconf_change_set_check_value (changeset, DOUBLE_CLICK_KEY, &value))) 
 		double_click_time = gconf_client_get_int (gconf_client_get_default (), DOUBLE_CLICK_KEY, NULL);
 	else
 		double_click_time = gconf_value_get_int (value);
@@ -581,7 +594,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
 	
 	gconf_peditor_new_boolean
-		(changeset, "/desktop/gnome/peripherals/mouse/locate_pointer_id", WID ("locate_pointer_toggle"), NULL);
+		(changeset, "/desktop/gnome/peripherals/mouse/locate_pointer", WID ("locate_pointer_toggle"), NULL);
 	/* Motion page */
 	/* speed */
 	gconf_peditor_new_numeric_range
@@ -599,14 +612,14 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	gconf_peditor_new_numeric_range
 		(changeset, "/desktop/gnome/peripherals/mouse/motion_threshold", WID ("sensitivity_scale"),
 		 "conv-to-widget-cb", threshold_from_gconf,
-		 "conv-from-widget-cb", gconf_value_float_to_int,
+		 "conv-from-widget-cb", threshold_to_gconf,
 		 NULL);
 
 	/* DnD threshold */
 	gconf_peditor_new_numeric_range
 		(changeset, "/desktop/gnome/peripherals/mouse/drag_threshold", WID ("drag_threshold_scale"),
 		 "conv-to-widget-cb", threshold_from_gconf,
-		 "conv-from-widget-cb", gconf_value_float_to_int,
+		 "conv-from-widget-cb", threshold_to_gconf,
 		 NULL);
 
 	/* listen to cursors changing */
@@ -688,6 +701,8 @@ main (int argc, char **argv)
 			    GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 			    NULL);
 
+	activate_settings_daemon ();
+
 	client = gconf_client_get_default ();
 	gconf_client_add_dir (client, "/desktop/gnome/peripherals/mouse", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
@@ -700,7 +715,7 @@ main (int argc, char **argv)
 		setup_dialog (dialog, changeset);
 
 		dialog_win = gtk_dialog_new_with_buttons
-			(_("Mouse Properties"), NULL, -1,
+			(_("Mouse Properties"), NULL, 0,
 			 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			 NULL);
 
