@@ -38,9 +38,10 @@ enum
 
 static GConfEnumStringPair toolbar_style_enums[] = {
   { 0, "both" },
-  { 0, "both_horiz" },
-  { 1, "icons" },
-  { 2, "text" }
+  { 1, "both_horiz" },
+  { 2, "icons" },
+  { 3, "text" },
+  { -1, NULL },
 };
 
 static GConfValue *
@@ -92,7 +93,6 @@ create_dialog (void)
   GladeXML *dialog;
 
   dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-ui-properties.glade", "gnome_ui_properties_dialog", NULL);
-  /*  dialog = glade_xml_new ("gnome2-ui-properties.glade", "gnome_ui_properties_dialog", NULL);*/
   return dialog;
 }
 
