@@ -34,6 +34,8 @@
 #include "xsettings-manager.h"
 #include "gnome-settings-daemon.h"
 
+/*#include "gnome-settings-disk.h"*/
+#include "gnome-settings-font.h"
 #include "gnome-settings-xsettings.h"
 #include "gnome-settings-mouse.h"
 #include "gnome-settings-keyboard.h"
@@ -41,6 +43,7 @@
 #include "gnome-settings-sound.h"
 #include "gnome-settings-wm.h"
 #include "gnome-settings-accessibility-keyboard.h"
+#include "gnome-settings-screensaver.h"
 
 #include "GNOME_SettingsDaemon.h"
 
@@ -221,6 +224,9 @@ gnome_settings_daemon_new (void)
    * for us
    */
   client = gconf_client_get_default ();
+
+/*  gnome_settings_disk_init (client);*/
+  gnome_settings_font_init (client);
   gnome_settings_xsettings_init (client);
   gnome_settings_mouse_init (client);
   gnome_settings_keyboard_init (client);
@@ -228,7 +234,8 @@ gnome_settings_daemon_new (void)
   gnome_settings_sound_init (client);
   gnome_settings_wm_init (client);
   gnome_settings_accessibility_keyboard_init (client);
-  
+  gnome_settings_screensaver_init (client);
+
   for (list = directories; list; list = list->next)
     {
       GError *error = NULL;
@@ -255,12 +262,16 @@ gnome_settings_daemon_new (void)
     }
   
   gdk_window_add_filter (NULL, manager_event_filter, NULL);
+
+/*  gnome_settings_disk_load (client);*/
+  gnome_settings_font_load (client);
   gnome_settings_xsettings_load (client);
   gnome_settings_mouse_load (client);
   gnome_settings_sound_load (client);
   gnome_settings_background_load (client);
   gnome_settings_wm_load (client);
   gnome_settings_accessibility_keyboard_load (client);
+  gnome_settings_screensaver_load (client);
   
   return G_OBJECT (daemon);
 }
