@@ -1,5 +1,8 @@
+#include <config.h>
 #include <string.h>
 #include <stdio.h>
+#include <glib.h>
+#include <libgnome/gnome-i18n.h>
 
 #include "gnome-settings-daemon.h"
 #include "gnome-settings-xsettings.h"
@@ -21,6 +24,19 @@ struct _TranslationEntry
 };
 
 
+static void
+translate_bool_int (TranslationEntry *trans,
+		    GConfValue       *value)
+{
+  g_assert (value->type == trans->gconf_type);
+  
+  g_print ("setting %s %d\n", 
+	   trans->xsetting_name,
+	   gconf_value_get_bool (value));
+  xsettings_manager_set_int (manager, trans->xsetting_name,
+                             gconf_value_get_bool (value));
+}
+
 static void
 translate_int_int (TranslationEntry *trans,
                    GConfValue       *value)
@@ -49,20 +65,24 @@ translate_string_string (TranslationEntry *trans,
 }
 
 static TranslationEntry translations [] = {
-  { "/desktop/gnome/peripherals/mouse/double_click", "Net/DoubleClickTime", GCONF_VALUE_INT,    
-    translate_int_int },
-  { "/desktop/gnome/gtk-color-palette", "Gtk/ColorPalette", GCONF_VALUE_STRING,
-    translate_string_string },
-  { "/desktop/gnome/gtk-toolbar-style", "Gtk/ToolbarStyle", GCONF_VALUE_STRING,
-    translate_string_string },
-  { "/desktop/gnome/gtk-toolbar-icon-size", "Gtk/ToolbarIconSize", GCONF_VALUE_STRING,
-    translate_string_string },
-  { "/desktop/gnome/interface/gtk_theme", "Net/ThemeName", GCONF_VALUE_STRING,
-    translate_string_string },
-  { "/desktop/gnome/interface/gtk_key_theme", "Gtk/KeyThemeName", GCONF_VALUE_STRING,
-    translate_string_string },
-  { "/desktop/gnome/interface/font_name", "Gtk/FontName", GCONF_VALUE_STRING,
-    translate_string_string }
+  { "/desktop/gnome/peripherals/mouse/double_click",	"Net/DoubleClickTime",
+      GCONF_VALUE_INT,		translate_int_int },
+  { "/desktop/gnome/gtk-color-palette",			"Gtk/ColorPalette",
+      GCONF_VALUE_STRING,	translate_string_string },
+  { "/desktop/gnome/interface/font_name",		"Gtk/FontName",
+      GCONF_VALUE_STRING,	translate_string_string },
+  { "/desktop/gnome/interface/gtk_key_theme",		"Gtk/KeyThemeName",
+      GCONF_VALUE_STRING,	translate_string_string },
+  { "/desktop/gnome/gtk-toolbar-style",			"Gtk/ToolbarStyle",
+      GCONF_VALUE_STRING,	translate_string_string },
+  { "/desktop/gnome/gtk-toolbar-icon-size",		"Gtk/ToolbarIconSize",
+      GCONF_VALUE_STRING,	translate_string_string },
+  { "/desktop/gnome/interface/cursor_blink",		"Net/CursorBlink",
+      GCONF_VALUE_BOOL,		translate_bool_int },
+  { "/desktop/gnome/interface/cursor_blink_time",	"Net/CursorBlinkTime",
+      GCONF_VALUE_INT,		translate_int_int },
+  { "/desktop/gnome/interface/gtk_theme",		"Net/ThemeName",
+      GCONF_VALUE_STRING,	translate_string_string }
 };
 
 static TranslationEntry*
@@ -134,7 +154,7 @@ process_value (TranslationEntry *trans,
         }
       else
         {
-          g_warning ("GConf key %s set to type %s but its expected type was %s\n",
+          g_warning (_("GConf key %s set to type %s but its expected type was %s\n"),
                      trans->gconf_key,
                      type_to_string (val->type),
                      type_to_string (trans->gconf_type));
