@@ -8,7 +8,7 @@
 #include <dirent.h>
 #include <libgnome/libgnome.h>
 
-#define ensure_category(category) ((category && category[0]) ? category : "gnome")
+#define ensure_category(category) ((category && category[0]) ? category : "gnome-2")
 
 static GtkObjectClass *sound_properties_parent_class;
 
@@ -29,7 +29,7 @@ enum
 	LAST_SIGNAL
 };
 
-guint sound_properties_signals[LAST_SIGNAL] = { 0 };
+static guint sound_properties_signals[LAST_SIGNAL] = { 0 };
 
 typedef struct 
 {
@@ -369,6 +369,14 @@ sound_properties_add_file (SoundProperties *props, gchar *filename,
 	
 	if (!category)
 		return;
+
+	/* Prefer the GNOME-2 versions, no need for dups though */
+	if (!g_ascii_strcasecmp (category, "gnome") ||
+	    !g_ascii_strcasecmp (category, "gtk-events"))
+	{
+		g_free (category);
+		return;
+	}
 	
 	prefix = g_strconcat ("=", filename, "=", NULL);
 	
@@ -512,17 +520,38 @@ sound_properties_save (SoundProperties *props, gchar *directory, gboolean save_t
 	{
 		SoundEvent *event = g_ptr_array_index (props->priv->events, i);
 		gchar *path;
+		const gchar *category;
 		
 		if (!(event->modified || (save_theme && event->theme)))
 			continue;
 
+		category = ensure_category (event->category);
+		
 		path = g_strdup_printf ("=%s/sound/events/%s.soundlist=/%s/file",
 			directory,
-			ensure_category (event->category),
+			category,
 			event->name);
 		gnome_config_set_string (path,
 					 (event->file) ? event->file : "");
 		g_free (path);
+
+		/* Bah, have to make GNOME1 copies */
+		if (!g_ascii_strcasecmp (category, "gnome-2") ||
+		    !g_ascii_strcasecmp (category, "gtk-events-2"))
+		{
+			gchar *other = g_strndup (category, strlen (category) - 2);
+			gchar *base = strip_ext (directory, ".gnome2");
+			if (base)
+			{
+				gchar *gnome1 = g_build_filename (base, ".gnome", NULL);
+				path = g_strdup_printf ("=%s/sound/events/%s.soundlist=/%s/file", gnome1, other, event->name);
+				gnome_config_set_string (path, (event->file) ? event->file : "");
+				g_free (path);
+				g_free (gnome1);
+				g_free (base);
+			}
+			g_free (other);
+		}
 	}
 
 	gnome_config_sync ();
