@@ -47,8 +47,12 @@ sound_view_destroy (GtkObject *object)
 		gtk_tree_path_free ((GtkTreePath *) view->priv->paths->data);
 		view->priv->paths = g_slist_remove (view->priv->paths, view->priv->paths->data);
 	}
-	g_hash_table_destroy (view->priv->event2path);
+	if (view->priv->event2path != NULL) {
+		g_hash_table_destroy (view->priv->event2path);
+		view->priv->event2path = NULL;
+	}
 	g_free (view->priv);
+	view->priv = NULL;
 
 	if (GTK_OBJECT_CLASS (sound_view_parent_class)->destroy)
 		GTK_OBJECT_CLASS (sound_view_parent_class)->destroy (object);
